@@ -1,7 +1,7 @@
 # coding: utf-8
 
 from os import path
-from typing import Union
+from typing import *
 
 from com import *
 from fastpip import *
@@ -126,6 +126,10 @@ class PackageManagerWindow(Ui_package_manager, QMainWindow):
             self.config.output_side = self.__output.linkage
         self.config.save_config()
 
+    def keyPressEvent(self, event: QKeyEvent):
+        if event.key() == Qt.Key_Escape:
+            self.close()
+
     def show_loading(self, text):
         self.lb_loading_tip.clear()
         self.lb_loading_tip.setText(text)
@@ -154,21 +158,54 @@ class PackageManagerWindow(Ui_package_manager, QMainWindow):
             self.__sort_by_column
         )
         self.tw_installed_info.clicked.connect(self.__show_label_selected_num)
-        self.le_search_pkgs_kwd.textChanged.connect(self.search_pkg_name_by_kwd)
-        self.lw_env_list.customContextMenuRequested.connect(
+        self.le_search_pkgs_kwd.textChanged.connect(self.search_pkgname_using_keyword)
+        self.lw_env_list.customContextMenuRequested[QPoint].connect(
             self.environlist_contextmenu
         )
-        self.tw_installed_info.customContextMenuRequested.connect(
+        self.tw_installed_info.customContextMenuRequested[QPoint].connect(
             self.packagesinfo_contextmenu
         )
         self.uiPushButton_show_output.clicked.connect(self.show_hide_output)
 
-    def open_selected_envfolder(self):
+    def selected_envfolder(self):
         index = self.lw_env_list.currentRow()
         environ_path = self.env_list[index].env_path
         if not path.isdir(environ_path):
             return MessageBox("提示", "所选环境目录不存在！").exec_()
-        open_explorer(environ_path, "root")
+        launch_explorer(environ_path)
+
+    def export_packages_info(self):
+        index = self.lw_env_list.currentRow()
+        if index == -1:
+            return
+        environ = self.env_list[index]
+        if not environ.pip_ready:
+            return MessageBox("提示", "所选环境不是有效环境！").exec_()
+        fullpath = QFileDialog.getSaveFileName(
+            self,
+            "选择文件夹",
+            path.join(self.config.last_path, "requirements.txt"),
+            "文本文件 (*.txt)",
+        )[0]
+        if not fullpath:
+            return
+        dir_path, name = path.split(fullpath)
+        self.config.last_path = dir_path
+        self.setCursor(Qt.WaitCursor)
+        thread_export = QThreadModel(environ.freeze, dir_path, name, no_path=True)
+        thread_export.after_completion(lambda: self.setCursor(Qt.ArrowCursor))
+        thread_export.start()
+        self.thread_repo.put(thread_export, 1)
+
+    def copy_environment_path(self):
+        index = self.lw_env_list.currentRow()
+        if index == -1:
+            return
+        environment = self.env_list[index]
+        if not environment.env_path:
+            return MessageBox("提示", "无效的 Python 环境！").exec_()
+        clipboard = QApplication.clipboard()
+        clipboard.setText(environment.env_path)
 
     def query_names(self):
         if self.selected_index == -1:
@@ -187,33 +224,45 @@ class PackageManagerWindow(Ui_package_manager, QMainWindow):
         if pkg_name is not None:
             query_window.start_query_name()
 
-    def environlist_contextmenu(self):
+    def environlist_contextmenu(self, point: QPoint):
+        if not self.lw_env_list.itemAt(point):
+            return
+
         contextmenu = QMenu(self)
 
-        action = QAction(QIcon(":/openfd.png"), "打开", self)
-        action.triggered.connect(self.open_selected_envfolder)
+        action = QAction(QIcon(":/openfd.png"), "打开目录", self)
+        action.triggered.connect(self.selected_envfolder)
+        contextmenu.addAction(action)
+
+        action = QAction(QIcon(":/copy.png"), "复制路径", self)
+        action.triggered.connect(self.copy_environment_path)
+        contextmenu.addAction(action)
+
+        action = QAction(QIcon(":/export.png"), "导出包列表", self)
+        action.triggered.connect(self.export_packages_info)
         contextmenu.addAction(action)
 
         contextmenu.addSeparator()
 
-        action = QAction(QIcon(":/add.png"), "添加", self)
-        action.triggered.connect(self.add_environ_manully)
-        contextmenu.addAction(action)
-
-        action = QAction(QIcon(":/search.png"), "搜索", self)
+        action = QAction(QIcon(":/search.png"), "自动搜索", self)
         action.triggered.connect(self.auto_search_environ)
         contextmenu.addAction(action)
 
-        contextmenu.addSeparator()
+        action = QAction(QIcon(":/add.png"), "手动添加", self)
+        action.triggered.connect(self.add_environ_manully)
+        contextmenu.addAction(action)
 
-        action = QAction(QIcon(":/delete.png"), "移除", self)
+        action = QAction(QIcon(":/delete.png"), "移除环境", self)
         action.triggered.connect(self.del_selected_environ)
         contextmenu.addAction(action)
 
         contextmenu.setStyleSheet("QMenu {padding: 10px; border: 1px solid black}")
         contextmenu.exec_(QCursor.pos())
 
-    def packagesinfo_contextmenu(self):
+    def packagesinfo_contextmenu(self, point: QPoint):
+        if not self.tw_installed_info.itemAt(point):
+            return
+
         contextmenu = QMenu(self)
         action_list = list()
 
@@ -227,6 +276,11 @@ class PackageManagerWindow(Ui_package_manager, QMainWindow):
         action_list.append(action)
         contextmenu.addAction(action)
 
+        action = QAction(QIcon(":/reinstall.png"), "强制重装", self)
+        action.triggered.connect(self.force_reinstall)
+        action_list.append(action)
+        contextmenu.addAction(action)
+
         action = QAction(QIcon(":/query.png"), "查询", self)
         action.triggered.connect(self.query_names)
         action_list.append(action)
@@ -270,7 +324,7 @@ class PackageManagerWindow(Ui_package_manager, QMainWindow):
                 return keyword in string_long
             return letters_following in string_long[1:]
 
-    def search_pkg_name_by_kwd(self):
+    def search_pkgname_using_keyword(self):
         keyword = self.le_search_pkgs_kwd.text()
         if not keyword:
             for i in range(self.tw_installed_info.rowCount()):
@@ -303,7 +357,7 @@ class PackageManagerWindow(Ui_package_manager, QMainWindow):
         cur_py_env_index = self.lw_env_list.currentRow()
         self.lw_env_list.clear()
         for env in self.env_list:
-            item = QListWidgetItem(str(env))
+            item = QListWidgetItem(QIcon(":/python.png"), str(env))
             item.setSizeHint(row_size)
             self.lw_env_list.addItem(item)
         if cur_py_env_index != -1:
@@ -384,11 +438,11 @@ class PackageManagerWindow(Ui_package_manager, QMainWindow):
 
         thread_get_pkgs_info = QThreadModel(do_get_pkgs_info)
         if not no_connect:
-            thread_get_pkgs_info.at_start(
+            thread_get_pkgs_info.before_starting(
                 self.lock_widgets,
                 lambda: self.show_loading("正在加载包信息..."),
             )
-            thread_get_pkgs_info.at_finish(
+            thread_get_pkgs_info.after_completion(
                 self.table_widget_pkgs_info_update,
                 self.hide_loading,
                 self.release_widgets,
@@ -429,11 +483,11 @@ class PackageManagerWindow(Ui_package_manager, QMainWindow):
 
         path_list_lower = [p.lower() for p in self.config.pypaths]
         thread_search_envs = QThreadModel(search_environ)
-        thread_search_envs.at_start(
+        thread_search_envs.before_starting(
             self.lock_widgets,
             lambda: self.show_loading("正在搜索 Python 安装目录..."),
         )
-        thread_search_envs.at_finish(
+        thread_search_envs.after_completion(
             self.environ_changed_clear_pkgs,
             self.list_widget_pyenvs_update,
             self.hide_loading,
@@ -498,11 +552,11 @@ class PackageManagerWindow(Ui_package_manager, QMainWindow):
                 ] = outdated_info[2]
 
         thread_get_outdated = QThreadModel(do_get_outdated)
-        thread_get_outdated.at_start(
+        thread_get_outdated.before_starting(
             self.lock_widgets,
             lambda: self.show_loading("正在检查更新..."),
         )
-        thread_get_outdated.at_finish(
+        thread_get_outdated.after_completion(
             self.table_widget_pkgs_info_update,
             self.hide_loading,
             self.release_widgets,
@@ -542,39 +596,40 @@ class PackageManagerWindow(Ui_package_manager, QMainWindow):
         ):
             widget.setEnabled(True)
 
-    def install_packages(self, cur_env, tobe_installed):
-        if not (cur_env and tobe_installed):
+    def install_packages(self, cur_env: PyEnv, package_names: List[str]):
+        if not (cur_env and package_names):
             return
         include_pre = self.config.include_pre
         user = self.config.install_for_user
+        force_reinstall = self.config.force_reinstall
         use_index_url = self.config.use_index_url
         index_url = self.config.index_url if use_index_url else ""
 
         def do_install():
-            installed = [
-                [name, result]
-                for name, result in loop_install(
-                    cur_env,
-                    tobe_installed,
+            installed_pkgs = list()
+            for package_name in package_names:
+                ins, res = cur_env.install(
+                    package_name,
                     pre=include_pre,
                     user=user,
+                    force_reinstall=force_reinstall,
                     index_url=index_url,
                 )
-            ]
-            separated = parse_package_names(i[0] for i in installed)
-            for i, value in enumerate(installed):
+                installed_pkgs.append([ins[0], res])
+            separated = parse_package_names(i[0] for i in installed_pkgs)
+            for i, value in enumerate(installed_pkgs):
                 value[0] = separated[i]
-            for name, result in installed:
+            for name, result in installed_pkgs:
                 item = self.__cur_pkgs_info.setdefault(name, ["", "", ""])
                 item[0] = "N/A"
                 item[2] = "安装成功" if result else "安装失败"
 
         thread_install_pkgs = QThreadModel(do_install)
-        thread_install_pkgs.at_start(
+        thread_install_pkgs.before_starting(
             self.lock_widgets,
             lambda: self.show_loading("正在安装..."),
         )
-        thread_install_pkgs.at_finish(
+        thread_install_pkgs.after_completion(
             self.table_widget_pkgs_info_update,
             self.hide_loading,
             self.release_widgets,
@@ -582,6 +637,47 @@ class PackageManagerWindow(Ui_package_manager, QMainWindow):
         thread_install_pkgs.start()
         self.thread_repo.put(thread_install_pkgs, 0)
 
+    def force_reinstall(self):
+        packages_names = self.package_names_selected()
+        if not packages_names:
+            if not packages_names:
+                MessageBox("提示", "没有选中任何项！").exec_()
+            return
+        cur_env = self.env_list[self.lw_env_list.currentRow()]
+        force_reinstall = QMessageBox(
+            QMessageBox.Question,
+            "重装",
+            "所选包的依赖包也会被重新安装为符合要求的版本，确定强制重装？",
+        )
+        force_reinstall.addButton("确定", QMessageBox.AcceptRole)
+        reject = force_reinstall.addButton("取消", QMessageBox.RejectRole)
+        force_reinstall.setDefaultButton(reject)
+        if force_reinstall.exec_() != 0:
+            return
+
+        def do_force_reinstall():
+            for package_name in packages_names:
+                item = self.__cur_pkgs_info.setdefault(package_name, ["", "", ""])
+                if item[0]:  # 当前版本号
+                    package_name = f"{package_name}=={item[0]}"
+                pkgnames, result = cur_env.install(package_name, force_reinstall=True)
+                if not result:
+                    item[0] = "N/A"
+                item[2] = "安装成功" if result else "安装失败"
+
+        thread_force_reinstall = QThreadModel(do_force_reinstall)
+        thread_force_reinstall.before_starting(
+            self.lock_widgets,
+            lambda: self.show_loading("正在重新安装..."),
+        )
+        thread_force_reinstall.after_completion(
+            self.table_widget_pkgs_info_update,
+            self.hide_loading,
+            self.release_widgets,
+        )
+        thread_force_reinstall.start()
+        self.thread_repo.put(thread_force_reinstall, 0)
+
     def uninstall_packages(self):
         pkgs_info_keys = tuple(self.__cur_pkgs_info.keys())
         pkg_indexs = self.indexs_of_selected_rows()
@@ -611,11 +707,11 @@ class PackageManagerWindow(Ui_package_manager, QMainWindow):
                 item[2] = "卸载成功" if code else "卸载失败"
 
         thread_uninstall_pkgs = QThreadModel(do_uninstall)
-        thread_uninstall_pkgs.at_start(
+        thread_uninstall_pkgs.before_starting(
             self.lock_widgets,
             lambda: self.show_loading("正在卸载..."),
         )
-        thread_uninstall_pkgs.at_finish(
+        thread_uninstall_pkgs.after_completion(
             self.table_widget_pkgs_info_update,
             self.hide_loading,
             self.release_widgets,
@@ -659,11 +755,11 @@ class PackageManagerWindow(Ui_package_manager, QMainWindow):
                     item[2] = "升级失败"
 
         thread_upgrade_pkgs = QThreadModel(do_upgrade)
-        thread_upgrade_pkgs.at_start(
+        thread_upgrade_pkgs.before_starting(
             self.lock_widgets,
             lambda: self.show_loading("正在升级..."),
         )
-        thread_upgrade_pkgs.at_finish(
+        thread_upgrade_pkgs.after_completion(
             self.table_widget_pkgs_info_update,
             self.hide_loading,
             self.release_widgets,
@@ -705,11 +801,11 @@ class PackageManagerWindow(Ui_package_manager, QMainWindow):
                 item[2] = "升级成功" if code else "升级失败"
 
         thread_upgrade_pkgs = QThreadModel(do_upgrade)
-        thread_upgrade_pkgs.at_start(
+        thread_upgrade_pkgs.before_starting(
             self.lock_widgets,
             lambda: self.show_loading("正在升级..."),
         )
-        thread_upgrade_pkgs.at_finish(
+        thread_upgrade_pkgs.after_completion(
             self.table_widget_pkgs_info_update,
             self.hide_loading,
             self.release_widgets,
@@ -772,6 +868,7 @@ class PackageInstallWindow(Ui_package_install, QMainWindow, QueryFilePath):
             self.le_use_index_url.setEnabled(True)
         else:
             self.le_use_index_url.setEnabled(False)
+        self.uiCheckBox_force_reinstall.setChecked(self.__parent.config.force_reinstall)
 
     def config_widgets_to_dict(self):
         self.package_names.extend(
@@ -782,6 +879,9 @@ class PackageInstallWindow(Ui_package_install, QMainWindow, QueryFilePath):
         self.__parent.config.install_for_user = self.cb_install_for_user.isChecked()
         self.__parent.config.index_url = self.le_use_index_url.text()
         self.__parent.config.use_index_url = self.cb_use_index_url.isChecked()
+        self.__parent.config.force_reinstall = (
+            self.uiCheckBox_force_reinstall.isChecked()
+        )
 
     def call_installpkg_back(self):
         self.close()  # 触发 closeEvent 更新配置
@@ -802,6 +902,10 @@ class PackageInstallWindow(Ui_package_install, QMainWindow, QueryFilePath):
         self.__save_window_size()
         self.config_widgets_to_dict()
 
+    def keyPressEvent(self, event: QKeyEvent):
+        if event.key() == Qt.Key_Escape:
+            self.close()
+
     def signal_slot_connection(self):
         self.pb_do_install.clicked.connect(self.call_installpkg_back)
         self.pb_save_as_text.clicked.connect(self.save_package_names)
@@ -833,13 +937,6 @@ class AddEnvironDialog(Ui_input_dialog, QMainWindow):
         self.uiPushButton_confirm.clicked.connect(self.__text_back)
         self.uiPushButton_select_envdir.clicked.connect(self.__select_envdir)
 
-    def keyPressEvent(self, event: QKeyEvent):
-        key = event.key()
-        if key == Qt.Key_Escape:
-            self.close()
-        elif key == Qt.Key_Enter or key == Qt.Key_Return:
-            self.__text_back()
-
     def __text_back(self):
         self.close()
         self.__callback(self.uiLineEdit_input_content.text())
@@ -852,6 +949,13 @@ class AddEnvironDialog(Ui_input_dialog, QMainWindow):
     def closeEvent(self, event: QCloseEvent):
         self.__save_window_size()
 
+    def keyPressEvent(self, event: QKeyEvent):
+        key = event.key()
+        if key == Qt.Key_Escape:
+            self.close()
+        elif key == Qt.Key_Enter or key == Qt.Key_Return:
+            self.__text_back()
+
     def __select_envdir(self):
         _path = QFileDialog.getExistingDirectory(self, "选择文件夹", self.last_path)
         if not _path:
@@ -889,13 +993,6 @@ class NameQueryPanel(Ui_query_panel, QMainWindow):
         self.signal_result.connect(self.uiPlainTextEdit_query_result.appendPlainText)
         self.uiPushButton_query.clicked.connect(self.start_query_name)
 
-    def keyPressEvent(self, event: QKeyEvent):
-        key = event.key()
-        if key == Qt.Key_Escape:
-            self.close()
-        elif key == Qt.Key_Enter or key == Qt.Key_Return:
-            self.start_query_name()
-
     def __save_query_configure(self):
         self.__parent.config.query_name = self.uiLineEdit_input_name.text()
         if self.uiRadioButton_pkg2import.isChecked():
@@ -910,6 +1007,13 @@ class NameQueryPanel(Ui_query_panel, QMainWindow):
     def closeEvent(self, event: QCloseEvent):
         self.__save_query_configure()
 
+    def keyPressEvent(self, event: QKeyEvent):
+        key = event.key()
+        if key == Qt.Key_Escape:
+            self.close()
+        elif key == Qt.Key_Enter or key == Qt.Key_Return:
+            self.start_query_name()
+
     def __start_querying(self):
         self.uiPushButton_query.setEnabled(False)
         self.uiRadioButton_pkg2import.setEnabled(False)
@@ -942,7 +1046,7 @@ class NameQueryPanel(Ui_query_panel, QMainWindow):
                 self.signal_result.emit(result)
 
         thread_query = QThreadModel(do_query)
-        thread_query.at_finish(self.__end_of_query)
+        thread_query.after_completion(self.__end_of_query)
         thread_query.start()
         self.__parent.thread_repo.put(thread_query, 1)
 
