@@ -2,13 +2,17 @@
 
 __doc__ = """包含AwesomePyKit的主要类、函数、配置文件路径等。"""
 
-import os.path as op
+import os.path as osp
 import re
-import subprocess
 from subprocess import *
+from typing import *
+from typing import Match
 
+import win32api
 from fastpip import PyEnv
 from PyQt5.QtCore import *
+# noinspection PyUnresolvedReferences
+from win32com.shell import shell
 
 
 def loop_install(
@@ -40,7 +44,7 @@ def clean_py_paths(paths):
 
 
 def check_index_url(url):
-    return bool(re.match(r"^https://.+/simple[/]?$", url.lower()))
+    return bool(re.match(r"^https://.+/simple/?$", url.lower()))
 
 
 def clean_index_urls(urls):
@@ -48,35 +52,36 @@ def clean_index_urls(urls):
 
 
 class QThreadModel(QThread):
-    def __init__(self, target, args=tuple()):
+    def __init__(self, target, *args, **kwargs):
         super().__init__()
-        self._args = args
-        self._target = target
-        self._at_start = list()
-        self._at_finish = list()
+        self.__target = target
+        self.__args = args
+        self.__kwargs = kwargs
+        self.__at_start = list()
+        self.__at_finish = list()
 
     def run(self):
-        self._target(*self._args)
+        self.__target(*self.__args, **self.__kwargs)
 
     def __repr__(self):
-        return f"{self._target} with args:{self._args}"
+        return f"{self.__target} with args: {self.__args}, kwargs: {self.__kwargs}"
 
     __str__ = __repr__
 
-    def at_start(self, *callable_objs):
+    def before_starting(self, *callable_objs):
         for cab in callable_objs:
             self.started.connect(cab)
-        self._at_start.extend(callable_objs)
+        self.__at_start.extend(callable_objs)
 
-    def at_finish(self, *callable_objs):
+    def after_completion(self, *callable_objs):
         for cab in callable_objs:
             self.finished.connect(cab)
-        self._at_finish.extend(callable_objs)
+        self.__at_finish.extend(callable_objs)
 
     def broken_signal(self):
-        for cab in self._at_start:
+        for cab in self.__at_start:
             self.started.disconnect(cab)
-        for cab in self._at_finish:
+        for cab in self.__at_finish:
             self.finished.disconnect(cab)
 
 
@@ -138,7 +143,9 @@ class ThreadRepo:
         return not self._thread_repo
 
 
-def get_cmd_out(*commands, regexp="", timeout=None):
+def get_cmd_out(
+    *commands, re_search=None, timeout=None
+) -> Union[str, Match[str], None]:
     """用于从cmd命令执行输出的字符匹配想要的信息。"""
     info = STARTUPINFO()
     info.dwFlags = STARTF_USESHOWWINDOW
@@ -148,26 +155,30 @@ def get_cmd_out(*commands, regexp="", timeout=None):
         strings, _ = proc.communicate(timeout=timeout)
     except Exception:
         return ""
-    if not regexp:
+    if not re_search:
         return strings.strip()
-    return re.search(regexp, strings)
+    return re.search(re_search, strings)
 
 
-def open_explorer(fd_path, option="root"):
-    """
-    option 参数：
-
-    为 root 表示以 fd_path 为根目录打开资源管理器
-
-    为 select 表示打开 fd_path 上一级目录并选中 fd_path 所指文件或目录
-    """
-    assert isinstance(fd_path, str)
-    assert option in ("root", "select")
-    fd_path = op.normpath(fd_path.strip())
-    if op.isfile(fd_path):
-        commands = f"explorer /select,{fd_path}"
-    elif op.isdir(fd_path):
-        commands = f"explorer /{option},{fd_path}"
-    else:
+def launch_explorer(folder_path: str, file_names: List[str] = None):
+    """使用资源管理器打开文件夹并选择文件名列表中的文件"""
+    assert isinstance(folder_path, str)
+    assert osp.isdir(folder_path)
+    assert file_names is None or all(isinstance(s, str) for s in file_names)
+    if file_names is None:
+        win32api.ShellExecute(0, "open", folder_path, None, None, 1)
         return
-    subprocess.run(commands)
+    folder_pidl = shell.SHILCreateFromPath(folder_path, 0)[0]
+    files_tobe_selected = list()
+    for file_name in file_names:
+        file_fullpath = osp.join(folder_path, file_name)
+        if not osp.isfile(file_fullpath):
+            continue
+        try:
+            file_id = shell.SHParseDisplayName(file_fullpath, 0)[0]
+            files_tobe_selected.append(file_id)
+        except Exception:
+            continue
+    if not files_tobe_selected:
+        return
+    shell.SHOpenFolderAndSelectItems(folder_pidl, files_tobe_selected, 0)
